import functools
from typing import Any, Callable, Dict, List, Literal

from werkflow.hooks.types.base.hook_types import HookType
from werkflow.hooks.types.base.registrar import registrar
from werkflow.prompt.types.base.base_prompt import BasePrompt

from .validator import StepHookCheckpoint, StepHookValidator


@registrar(HookType.STEP)
def step(
    *names, 
    prompts: List[BasePrompt]=[], 
    skip_on_fail: bool=False,
    condition: Callable[[
        Dict[str, Any]
    ], bool]=None,
    checkpoint: Dict[
        Literal[
            'serializer', 
            'path', 
            'action',
        ], 
        str | Literal[
            'load', 
            'save',
        ],
    ] | None = None,
):
    
    validated_checkpoint: StepHookCheckpoint | None = None
    if validated_checkpoint:
        validated_checkpoint = StepHookCheckpoint(**checkpoint)

    StepHookValidator(
        names=names,
        prompts=prompts,
        skip_on_fail=skip_on_fail,
        condition=condition,
        checkpoint=validated_checkpoint,
    )

    def wrapper(func):

        @functools.wraps(func)
        def decorator(*args, **kwargs):
            return func(*args, **kwargs)
        
        return decorator
    return wrapper


from typing import (
    Any,
    Callable,
    Dict,
    List,
    Literal,
    Optional,
    Tuple,
)

from pydantic import BaseModel, StrictBool, StrictStr

from werkflow.prompt.types.base.base_prompt import BasePrompt


class StepHookCheckpoint(BaseModel):
    serializer: Literal['json', 'bytes'] = 'json'
    action: Literal['load', 'save']
    path: StrictStr

class StepHookValidator(BaseModel):
    names: Tuple[StrictStr, ...]
    prompts: List[BasePrompt]=[]
    skip_on_fail: StrictBool
    condition: Optional[
        Callable[
            [Dict[str, Any]],
            bool
        ]
    ]=None
    checkpoint: StepHookCheckpoint | None = None

    class Config:
        arbitrary_types_allowed=True




from typing import (
    Any,
    Callable,
    Dict,
    List,
    Literal,
    Tuple,
)

from werkflow.hooks.types.base.base_hook import BaseHook
from werkflow.hooks.types.base.hook_types import HookType
from werkflow.prompt.types.base.base_prompt import BasePrompt


class StepHook(BaseHook):

    def __init__(
            self, name: str, 
            shortname: str, 
            call: Callable[..., Any], 
            *names: Tuple[str, ...],
            prompts: List[BasePrompt]=[],
            skip_on_fail: bool=False,
            condition: Callable[
                [Dict[str, Any]],
                bool
            ]=None,
            checkpoint: Dict[
                Literal[
                    'serializer', 
                    'path', 
                    'action',
                ], 
                str | Literal[
                    'load', 
                    'save',
                ],
            ] | None = None,
        ) -> None:

        super().__init__(
            name, 
            shortname,
            call, 
            *names,
            hook_type=HookType.STEP,
            prompts=prompts,
            skip_on_fail=skip_on_fail,
            condition=condition,
        )

        self.checkpoint = checkpoint
